@@ -30,6 +30,19 @@ algorithm <- function(x) {
   else x$algorithm
 }
 
+name_model <- function(family) {
+  # create the name of the fitted stan model
+  # Args:
+  #   family: A family object
+  if (!is(family, "family")) {
+    mn <- "brms-model"
+  } else {
+    type <- ifelse(is.null(family$type), "", paste(",", family$type))
+    mn <- paste0(family$family, "(", family$link, type, ") brms-model")
+  }
+  mn
+}
+
 restructure <- function(x, rstr_summary = FALSE) {
   # restructure old brmsfit objects to work with the latest brms version
   # Args:
@@ -41,14 +54,18 @@ restructure <- function(x, rstr_summary = FALSE) {
   }
   if (is.null(x$version)) {
     # this is the latest version without saving the version number
-    x$version <- package_version("0.9.1")
+    x$version <- list(brms = package_version("0.9.1"))
+  } else if (is.package_version(x$version)) {
+    # also added the rstan version in brms 1.5.0
+    x$version <- list(brms = x$version)
   }
-  if (x$version < utils::packageVersion("brms")) {
+  if (x$version$brms < utils::packageVersion("brms")) {
     # element 'nonlinear' deprecated as of brms > 0.9.1
     # element 'partial' deprecated as of brms > 0.8.0
-    x$formula <- SW(amend_formula(formula(x), data = model.frame(x),
-                                  family = family(x), partial = x$partial,
-                                  nonlinear = x$nonlinear))
+    x$formula <- SW(amend_formula(
+      formula(x), data = model.frame(x), family = family(x), 
+      partial = x$partial, nonlinear = x$nonlinear
+    ))
     x$nonlinear <- x$partial <- NULL
     x$formula[["old_mv"]] <- is_old_mv(x)
     bterms <- parse_bf(formula(x), family = family(x))
@@ -60,35 +77,35 @@ restructure <- function(x, rstr_summary = FALSE) {
       # deprecated as of brms 1.4.0
       class(x$autocor) <- "cor_fixed"
     }
-    if (x$version <= "0.9.1") {
+    if (x$version$brms <= "0.9.1") {
       # update gaussian("log") to lognormal() family
       nresp <- length(bterms$response)
-      if (is_old_lognormal(x$family, nresp = nresp, version = x$version)) {
+      if (is_old_lognormal(x$family, nresp = nresp, version = x$version$brms)) {
         object$family <- object$formula$family <- lognormal()
       }
     }
-    if (x$version <= "0.10.0.9000") {
-      if (length(bterms$nlpars)) {
+    if (x$version$brms <= "0.10.0.9000") {
+      if (length(bterms$auxpars$mu$nlpars)) {
         # nlpar and group have changed positions
-        change <- change_old_ranef(x$ranef, pars = parnames(x),
-                                   dims = x$fit@sim$dims_oi)
+        change <- change_old_re(x$ranef, pars = parnames(x),
+                                dims = x$fit@sim$dims_oi)
         x <- do_renaming(x, change)
       }
     }
-    if (x$version < "1.0.0") {
+    if (x$version$brms < "1.0.0") {
       # double underscores were added to group-level parameters
-      change <- change_old_ranef2(x$ranef, pars = parnames(x),
-                                  dims = x$fit@sim$dims_oi)
+      change <- change_old_re2(x$ranef, pars = parnames(x),
+                               dims = x$fit@sim$dims_oi)
       x <- do_renaming(x, change)
     }
-    if (x$version <= "1.0.1") {
+    if (x$version$brms <= "1.0.1") {
       # names of spline parameters had to be changed after
       # allowing for multiple covariates in one spline term
-      change <- change_old_splines(bterms, pars = parnames(x),
-                                   dims = x$fit@sim$dims_oi)
+      change <- change_old_sm(bterms, pars = parnames(x),
+                              dims = x$fit@sim$dims_oi)
       x <- do_renaming(x, change)
     }
-    if (x$version <= "1.2.0") {
+    if (x$version$brms <= "1.2.0") {
       x$ranef$type[x$ranef$type == "mono"] <- "mo"
       x$ranef$type[x$ranef$type == "cse"] <- "cs"
     }
@@ -132,7 +149,8 @@ link <- function(x, link) {
   switch(link, 
     "identity" = x, 
     "log" = log(x), 
-    "logm1" = logm1(x), 
+    "logm1" = logm1(x),
+    "log1p" = log1p(x),
     "inverse" = 1 / x,
     "sqrt" = sqrt(x), 
     "1/mu^2" = 1 / x^2, 
@@ -157,6 +175,7 @@ ilink <- function(x, link) {
     "identity" = x, 
     "log" = exp(x),
     "logm1" = expp1(x),
+    "log1p" = expm1(x),
     "inverse" = 1 / x,
     "sqrt" = x^2, 
     "1/mu^2" = 1 / sqrt(x), 
@@ -184,23 +203,29 @@ prepare_conditions <- function(x, conditions = NULL, effects = NULL,
   mf <- model.frame(x)
   new_formula <- update_re_terms(formula(x), re_formula = re_formula)
   bterms <- parse_bf(new_formula, family = family(x))
-  int_effects <- c(get_effect(bterms, "mo"), 
-                   rmNULL(bterms[c("trials", "cat")]))
+  int_effects <- c(
+    get_effect(bterms, "mo"), 
+    rmNULL(bterms$adforms[c("trials", "cat")])
+  )
   int_vars <- unique(ulapply(int_effects, all.vars))
   if (is.null(conditions)) {
-    if (!is.null(bterms$trials)) {
+    if (!is.null(bterms$adforms$trials)) {
       message("Using the median number of trials by default")
     }
     # list all required variables
-    random <- get_random(bterms)
-    req_vars <- c(lapply(get_effect(bterms), rhs), random$form, 
-                  lapply(random$gcall, "[[", "weightvars"),
-                  lapply(get_effect(bterms, "mo"), rhs),
-                  lapply(get_effect(bterms, "me"), rhs),
-                  lapply(get_effect(bterms, "gam"), rhs), 
-                  bterms[c("cs", "se", "disp", "trials", "cat")])
+    re <- get_re(bterms)
+    req_vars <- c(
+      lapply(get_effect(bterms, "fe"), rhs), 
+      lapply(get_effect(bterms, "mo"), rhs),
+      lapply(get_effect(bterms, "me"), rhs),
+      lapply(get_effect(bterms, "sm"), rhs),
+      lapply(get_effect(bterms, "cs"), rhs),
+      re$form, lapply(re$gcall, "[[", "weightvars"),
+      bterms$adforms[c("se", "disp", "trials", "cat")],
+      bterms$auxpars$mu$covars
+    )
     req_vars <- unique(ulapply(req_vars, all.vars))
-    req_vars <- setdiff(req_vars, c(rsv_vars, names(bterms$nlpars)))
+    req_vars <- setdiff(req_vars, rsv_vars)
     conditions <- as.data.frame(as.list(rep(NA, length(req_vars))))
     names(conditions) <- req_vars
     for (v in req_vars) {
@@ -567,19 +592,22 @@ get_auxpar <- function(x, i = NULL) {
   # get samples of an auxiliary parameter
   # Args:
   #   x: object to extract postarior samples from
-  #   data: data initially passed to Stan
   #   i: the current observation number
   #      (used in predict and log_lik)
   if (is.list(x)) {
     # compute auxpar in distributional regression models
-    ilink <- get(x[["ilink"]], mode = "function")
-    x <- ilink(get_eta(i = if (!is.null(i)) i else NULL, draws = x))
+    if (!x$f$family %in% auxpars()) {
+      # link functions may not just span the parameter
+      # and will be applied later on
+      x$f$link <- "identity"
+    }
+    x <- ilink(get_eta(x, i = i), x$f$link)
   } else {
-    if (!is.null(i) && isTRUE(ncol(x) > 1L)) {
+    if (!is.null(i) && is.matrix(x) && ncol(x) > 1L) {
       x <- x[, i, drop = FALSE]
     }
   }
-  if (is.null(i) && isTRUE(ncol(x) == 1L)) {
+  if (is.null(i) && is.matrix(x) && ncol(x) == 1L) {
     # for compatibility with fitted helper functions
     x <- as.vector(x)
   }
@@ -617,7 +645,7 @@ get_theta <- function(draws, i = NULL, par = c("zi", "hu")) {
   par <- match.arg(par)
   if (!is.null(draws$data$N_trait)) {
     j <- if (!is.null(i)) i else seq_len(draws$data$N_trait)
-    theta <- ilink(get_eta(draws, j + draws$data$N_trait), "logit")
+    theta <- ilink(get_eta(draws$mu, j + draws$data$N_trait), "logit")
   } else {
     theta <- get_auxpar(draws[[par]], i = i)
   }
@@ -687,7 +715,7 @@ prepare_family <- function(x) {
   # prepare for calling family specific log_lik / predict functions
   family <- family(x)
   nresp <- length(parse_bf(x$formula, family = family)$response)
-  if (is_old_lognormal(family, nresp = nresp, version = x$version)) {
+  if (is_old_lognormal(family, nresp = nresp, version = x$version$brms)) {
     family <- lognormal()
   } else if (is_linear(family) && nresp > 1L) {
     family$family <- paste0(family$family, "_mv")
