@@ -89,9 +89,9 @@ vcov.brmsfit <- function(object, correlation = FALSE, ...) {
 #' 
 #' @param object An object of class \code{brmsfit}.
 #' @param estimate The point estimate to be calculated 
-#'  for the random effects, either \code{"mean"} or \code{"median"}.
+#'  for the group-level effects, either \code{"mean"} or \code{"median"}.
 #' @param var logical; indicating if the covariance matrix 
-#'  for each random effects should be computed.
+#'  for each group-level effects should be computed.
 #' @param ... Further arguments to be passed to the function 
 #'  specified in \code{estimate}.
 #'
@@ -127,7 +127,7 @@ ranef.brmsfit <- function(object, estimate = c("mean", "median"),
   pars <- parnames(object)
   
   .ranef <- function(group, nlpar = "") {
-    # get random effects of a grouping factor
+    # get group-level effects of a grouping factor
     # Args:
     #   group: name of a grouping factor
     #   nlpar: name of a non-linear parameter
@@ -298,8 +298,8 @@ coef.brmsfit <- function(object, estimate = c("mean", "median"), ...) {
 #' Extract variance and correlation components
 #' 
 #' This function calculates the estimated standard deviations, 
-#' correlations and covariances of the random-effects terms 
-#' in a mixed-effects model of class \code{brmsfit}. 
+#' correlations and covariances of the group-level terms 
+#' in a multilevel model of class \code{brmsfit}. 
 #' For linear models, the residual standard deviations, 
 #' correlations and covariances are also returned. 
 #' 
@@ -330,11 +330,11 @@ coef.brmsfit <- function(object, estimate = c("mean", "median"), ...) {
 #' \dontrun{
 #' fit <- brm(count ~ log_Age_c + log_Base4_c * Trt_c + (1+Trt_c|visit), 
 #'              data = epilepsy, family = "poisson", chains = 1)
-#' ## return the means of random effects covariances
+#' ## return the means of group-level covariances
 #' (vc <- VarCorr(fit))
 #' as.data.frame(vc)
 #' 
-#' ## return 2.5% and 97.5% quantiles of random effects covariances
+#' ## return 2.5% and 97.5% quantiles of group-level covariances
 #' VarCorr(fit, estimate = "quantile", probs = c(0.025, 0.975))
 #' }
 #' 
@@ -420,12 +420,14 @@ VarCorr.brmsfit <- function(x, sigma = 1, estimate = "mean",
     p <- group <- NULL
   } 
   # special treatment of residuals variances in linear models
-  has_sigma <- has_sigma(family, bterms, autocor = x$autocor, incmv = TRUE)
+  has_sigma <- has_sigma(family, bterms, incmv = TRUE)
   if (has_sigma && !"sigma" %in% names(bterms$auxpars)) {
     cor_pars <- get_cornames(bterms$response, type = "rescor", 
                              brackets = FALSE)
-    p <- lc(p, list(rnames = bterms$response, cor_pars = cor_pars,
-                    sd_pars = c("sigma", paste0("sigma_", bterms$response))))
+    p <- lc(p, 
+      list(rnames = bterms$response, cor_pars = cor_pars,
+           sd_pars = c("sigma", paste0("sigma_", bterms$response)))
+    )
     group <- c(group, "RESIDUAL")
   } 
   VarCorr <- lapply(p, extract)
@@ -625,10 +627,6 @@ prior_samples.brmsfit <- function(x, pars = NA, parameters = NA, ...) {
 }
 
 #' Print a summary for a fitted model represented by a \code{brmsfit} object
-#'
-#' Print basic information regarding the fitted model and a summary 
-#' for the fixed and random effects
-#' estimated by the samples included in a \code{brmsfit} object.
 #' 
 #' @aliases print.brmssummary
 #' 
@@ -646,9 +644,6 @@ print.brmsfit <- function(x, digits = 2, ...) {
 }  
 
 #' Create a summary of a fitted model represented by a \code{brmsfit} object
-#' 
-#' Summarize estimated fixed and random effects as well as other useful
-#' results included in a \code{brmsfit} object.
 #'
 #' @param object An object of class \code{brmsfit}
 #' @param waic Logical; Indicating if the WAIC should be computed
@@ -720,9 +715,9 @@ summary.brmsfit <- function(object, waic = FALSE, priors = FALSE,
     }
     
     # fixed effects summary
-    fix_pars <- pars[grepl(fixef_pars(), pars)]
-    out$fixed <- fit_summary[fix_pars, , drop = FALSE]
-    rownames(out$fixed) <- gsub(fixef_pars(), "", fix_pars)
+    fe_pars <- pars[grepl(fixef_pars(), pars)]
+    out$fixed <- fit_summary[fe_pars, , drop = FALSE]
+    rownames(out$fixed) <- gsub(fixef_pars(), "", fe_pars)
     
     # summary of family specific parameters
     is_mv_par <- apply(sapply(c("^sigma_", "^rescor_"), grepl, pars), 1, any)
@@ -759,7 +754,7 @@ summary.brmsfit <- function(object, waic = FALSE, priors = FALSE,
       rownames(out$random[[g]]) <- c(sd_names, cor_names)
     }
     
-    # summary of splines
+    # summary of smooths
     spline_pars <- pars[grepl("^sds_", pars)]
     if (length(spline_pars)) {
       out$splines <- fit_summary[spline_pars, , drop = FALSE]
@@ -776,11 +771,11 @@ nsamples.brmsfit <- function(x, subset = NULL,
   if (!is(x$fit, "stanfit") || !length(x$fit@sim)) {
     out <- 0
   } else {
-    ntsamples <- x$fit@sim$iter
+    ntsamples <- x$fit@sim$n_save[1]
     if (!incl_warmup) {
-      ntsamples <- ntsamples - x$fit@sim$warmup
+      ntsamples <- ntsamples - x$fit@sim$warmup2[1]
     }
-    ntsamples <- ceiling(ntsamples / x$fit@sim$thin * x$fit@sim$chains)
+    ntsamples <- ntsamples * x$fit@sim$chains
     if (length(subset)) {
       out <- length(subset)
       if (out > ntsamples || max(subset) > ntsamples) {
@@ -905,9 +900,8 @@ launch_shiny.brmsfit <- function(x, rstudio = getOption("shinystan.rstudio"),
 #' fit <- brm(count ~ log_Age_c + log_Base4_c * Trt_c 
 #'            + (1|patient) + (1|visit), 
 #'            data = epilepsy, family = "poisson")
-#' ## plot fixed effects as well as standard devations of the random effects
 #' plot(fit)
-#' ## plot fixed effects only
+#' ## plot population-level effects only
 #' plot(fit, pars = "^b_") 
 #' }
 #' 
@@ -1256,7 +1250,7 @@ marginal_effects.brmsfit <- function(x, effects = NULL, conditions = NULL,
              "assumption for family ", x$family$family, ".")
   }
   rsv_vars <- rsv_vars(x$family, nresp = length(bterms$response),
-                       rsv_intercept = attr(bterms$fixed, "rsv_intercept"),
+                       rsv_intercept = attr(bterms$fe, "rsv_intercept"),
                        old_mv = attr(bterms$formula, "old_mv"))
   if (is.null(effects)) {
     effects <- get_all_effects(bterms, rsv_vars = rsv_vars)
@@ -1357,7 +1351,7 @@ marginal_effects.brmsfit <- function(x, effects = NULL, conditions = NULL,
     attr(marg_res, "effects") <- effects[[i]]
     attr(marg_res, "surface") <- both_numeric && surface
     point_args <- nlist(mf, effects = effects[[i]], conditions,
-                        groups = get_random(bterms)$group, family = x$family)
+                        groups = get_re(bterms)$group, family = x$family)
     attr(marg_res, "points") <- do.call(make_point_frame, point_args)
     results[[paste0(effects[[i]], collapse = ":")]] <- marg_res
   }
@@ -1377,27 +1371,33 @@ marginal_smooths.brmsfit <- function(x, smooths = NULL,
   conditions <- prepare_conditions(x)
   smooths <- rename(as.character(smooths), " ", "")
   bterms <- parse_bf(formula(x), family = family(x))
-  if (length(bterms$nlpars)) {
-    lee <- bterms$nlpars
-  } else {
-    lee <- named_list(bterms$response, list(bterms))
-    if (length(lee) == 1L) {
-      names(lee) <- ""
+  lee <- list()
+  if (length(bterms$response) > 1L) {
+    for (r in bterms$response) {
+      lee <- c(lee, setNames(bterms$auxpars["mu"], r))
+    }
+    bterms$auxpars[["mu"]] <- NULL
+  }
+  for (ap in names(bterms$auxpars)) {
+    bt <- bterms$auxpars[ap]
+    if (is.btnl(bt[[1]])) {
+      lee <- c(lee, bt[[1]]$nlpars)
+    } else {
+      lee <- c(lee, bt)
     }
   }
-  lee <- c(lee, bterms$auxpars)
-  
-  args <- nlist(x, smooths_only = TRUE, allow_new_levels = TRUE,
-                incl_autocor = FALSE, f = prepare_family(x), 
-                nsamples = nsamples(x))
+  args <- nlist(
+    fit = x, smooths_only = TRUE, nsamples = nsamples(x),
+    incl_autocor = FALSE, allow_new_levels = TRUE
+  )
   too_many_covars <- FALSE
   results <- list()
   for (k in seq_along(lee)) {
     # loop over elements that may contain smooth terms
-    splines <- get_spline_labels(lee[[k]])
-    splines_by <- get_spline_labels(lee[[k]], data = mf)
-    covars <- get_spline_labels(lee[[k]], covars = TRUE, combine = FALSE)
-    for (i in seq_along(splines)) {
+    sm_labels <- get_sm_labels(lee[[k]])
+    sm_labels_by <- get_sm_labels(lee[[k]], data = mf)
+    covars <- get_sm_labels(lee[[k]], covars = TRUE, combine = FALSE)
+    for (i in seq_along(sm_labels)) {
       # loop over smooth terms and compute their predictions
       covars_no_by_factor <- covars[[i]]
       byvars <- attr(covars, "byvars")[[i]]
@@ -1408,8 +1408,8 @@ marginal_smooths.brmsfit <- function(x, smooths = NULL,
       if (ncovars > 2L) {
         too_many_covars <- TRUE
       }
-      include_spline <- !length(smooths) || splines[[i]] %in% smooths
-      if (include_spline && ncovars <= 2L) {
+      include_smooth <- !length(smooths) || sm_labels[[i]] %in% smooths
+      if (include_smooth && ncovars <= 2L) {
         values <- named_list(covars[[i]])
         for (cv in names(values)) {
           if (is.numeric(mf[[cv]])) {
@@ -1427,29 +1427,29 @@ marginal_smooths.brmsfit <- function(x, smooths = NULL,
             g2 = newdata[[covars_no_byfactor[2]]], 
             d1 = x$data[, covars_no_byfactor[1]],
             d2 = x$data[, covars_no_byfactor[2]],
-            dist = too_far)
+            dist = too_far
+          )
           newdata <- newdata[!ex_too_far, ]  
         }
         other_vars <- setdiff(names(conditions), covars[[i]])
         newdata[, other_vars] <- conditions[1, other_vars]
         # prepare draws for linear_predictor
-        more_args <- nlist(newdata, nlpar = names(lee)[k], 
-                           rhs_formula = lee[[k]]$formula)
-        draws <- do.call(.extract_draws, c(args, more_args))
-        J <- which(attr(splines_by, "termnum") == i)
+        more_args <- nlist(x = lee[[k]], newdata, nlpar = names(lee)[k])
+        draws <- do.call(extract_draws, c(args, more_args))
+        J <- which(attr(sm_labels_by, "termnum") == i)
         scs <- unlist(attr(draws$data[["X"]], "smooth_cols")[J])
         draws$data[["X"]] <- draws$data[["X"]][, scs, drop = FALSE]
         draws[["b"]] <- draws[["b"]][, scs, drop = FALSE]
         draws[["Zs"]] <- draws[["Zs"]][J] 
         draws[["s"]] <- draws[["s"]][J]
-        eta <- get_eta(i = NULL, draws = draws)
+        eta <- get_eta(draws = draws, i = NULL)
         eta <- get_summary(eta, robust = TRUE, probs = probs)
         colnames(eta) <- c("estimate__", "se__", "lower__", "upper__")
         res <- cbind(newdata[, covars[[i]], drop = FALSE], eta)
         if (length(byfactors)) {
           res$cond__ <- Reduce(paste_colon, res[, byfactors, drop = FALSE]) 
         }
-        response <- splines[[i]]
+        response <- sm_labels[[i]]
         if (isTRUE(nzchar(names(lee)[k]))) {
           response <- paste0(names(lee)[k], ": ", response)
         }
@@ -1568,7 +1568,7 @@ marginal_smooths.brmsfit <- function(x, smooths = NULL,
 #' pp <- predict(fit)
 #' head(pp)
 #' 
-#' ## predicted responses excluding the random effect of age
+#' ## predicted responses excluding the group-level effect of age
 #' pp2 <- predict(fit, re_formula = ~ (1|patient))
 #' head(pp2)
 #' 
@@ -1589,10 +1589,14 @@ predict.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
                             probs = c(0.025, 0.975), ...) {
   contains_samples(object)
   object <- restructure(object)
-  draws_args <- nlist(x = object, newdata, re_formula, incl_autocor, 
-                      allow_new_levels, subset, nsamples)
+  draws_args <- nlist(
+    x = object, newdata, re_formula, incl_autocor, 
+    allow_new_levels, subset, nsamples
+  )
   draws <- do.call(extract_draws, draws_args)
-  draws$eta <- get_eta(i = NULL, draws = draws)
+  if (is.list(draws$mu[["mv"]])) {
+    draws$mu <- get_eta(draws$mu)
+  }
   for (ap in intersect(auxpars(), names(draws))) {
     if (is.list(draws[[ap]])) {
       draws[[ap]] <- get_auxpar(draws[[ap]])
@@ -1605,8 +1609,9 @@ predict.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
        else if (!is.null(draws$data$N_tg)) draws$data$N_tg
        else if (is.cor_fixed(draws$autocor)) 1
        else draws$data$N
-  out <- do.call(cbind, lapply(seq_len(N), predict_fun, 
-                               draws = draws, ntrys = ntrys))
+  out <- do.call(cbind, 
+    lapply(seq_len(N), predict_fun, draws = draws, ntrys = ntrys)
+  )
   # percentage of invalid samples for truncated discrete models
   # should always be zero for all other models; see predict.R
   pct_invalid <- get_pct_invalid(out, lb = draws$data$lb, ub = draws$data$ub) 
@@ -1676,6 +1681,8 @@ posterior_predict.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
 #'  If \code{"response"} results are returned on the scale 
 #'  of the response variable. If \code{"linear"} 
 #'  fitted values are returned on the scale of the linear predictor.
+#' @param auxpar Optional name of a predicted auxiliary parameter.
+#'  If specified, fitted values of this parameters are returned.
 #'
 #' @return Fitted values extracted from \code{object}. 
 #'  The output depends on the family:
@@ -1710,41 +1717,60 @@ posterior_predict.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
 fitted.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
                            scale = c("response", "linear"),
                            allow_new_levels = FALSE, incl_autocor = TRUE,
-                           subset = NULL, nsamples = NULL, sort = FALSE,
-                           summary = TRUE, robust = FALSE,
+                           auxpar = NULL, subset = NULL, nsamples = NULL, 
+                           sort = FALSE, summary = TRUE, robust = FALSE,
                            probs = c(0.025, 0.975), ...) {
   scale <- match.arg(scale)
   contains_samples(object)
   object <- restructure(object)
-  draws_args <- nlist(x = object, newdata, re_formula, incl_autocor, 
-                      allow_new_levels, subset, nsamples)
+  draws_args <- nlist(
+    x = object, newdata, re_formula, incl_autocor, 
+    allow_new_levels, subset, nsamples
+  )
   draws <- do.call(extract_draws, draws_args)
-  # get mu and scale it appropriately
-  mu <- get_eta(i = NULL, draws = draws)
-  if (grepl("_mv$", draws$f$family) && !is.null(draws[["mv"]])) {
-    # collapse over responses in linear MV models
-    dim(mu) <- c(dim(mu)[1], prod(dim(mu)[2:3]))
-  }
-  for (ap in intersect(auxpars(), names(draws))) {
-    if (is(draws[[ap]], "list")) {
-      draws[[ap]] <- get_auxpar(draws[[ap]])
+  auxpars <- intersect(auxpars(), names(draws))
+  if (!length(auxpar)) {
+    if (is.list(draws$mu[["mv"]])) {
+      draws$mu <- get_eta(draws$mu)
     }
-  }
-  if (scale == "response") {
-    # original families are required for fitted helper functions 
-    draws$f <- family(object)
-    fitted_fun <- paste0("fitted_", draws$f$family)
-    fitted_fun <- get(fitted_fun, asNamespace("brms"))
-    mu <- do.call(fitted_fun, nlist(mu, draws))
+    for (ap in auxpars) {
+      if (is.list(draws[[ap]])) {
+        draws[[ap]] <- get_auxpar(draws[[ap]])
+      }
+    }
+    if (grepl("_mv$", draws$f$family) && length(dim(draws$mu)) == 3L) {
+      # collapse over responses in linear MV models
+      dim(draws$mu) <- c(dim(draws$mu)[1], prod(dim(draws$mu)[2:3]))
+    }
+    if (scale == "response") {
+      # original families are required for fitted helper functions 
+      draws$f <- family(object)
+      fitted_fun <- paste0("fitted_", draws$f$family)
+      fitted_fun <- get(fitted_fun, asNamespace("brms"))
+      draws$mu <- fitted_fun(draws)
+    }
+  } else {
+    auxpars <- setdiff(auxpars, "mu")
+    if (length(auxpar) != 1L || !auxpar %in% auxpars) {
+      stop2("Invalid argument 'auxpar'. Valid auxiliary ",
+            "parameters are: ", collapse_comma(auxpars))
+    }
+    if (!is.list(draws[[auxpar]])) {
+      stop2("Auxiliary parameter '", auxpar, "' was not predicted.")
+    }
+    if (scale == "linear") {
+      draws[[auxpar]]$f$link <- "identity"
+    }
+    draws$mu <- get_auxpar(draws[[auxpar]])
   }
   old_order <- attr(draws$data, "old_order")
-  out <- reorder_obs(mu, old_order, sort = sort)
-  colnames(out) <- NULL
+  draws$mu <- reorder_obs(draws$mu, old_order, sort = sort)
+  colnames(draws$mu) <- NULL
   if (summary) {
-    mu <- get_summary(mu, probs = probs, robust = robust)
-    rownames(mu) <- seq_len(nrow(mu))
+    draws$mu <- get_summary(draws$mu, probs = probs, robust = robust)
+    rownames(draws$mu) <- seq_len(nrow(draws$mu))
   }
-  mu
+  draws$mu
 }
 
 #' Extract Model Residuals from brmsfit Objects
@@ -1811,9 +1837,10 @@ residuals.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
     stop2("Residuals not implemented for family '", family$family, "'.")
   }
   
-  standata <- amend_newdata(newdata, fit = object, re_formula = re_formula,
-                            allow_new_levels = allow_new_levels, 
-                            check_response = TRUE)
+  standata <- amend_newdata(
+    newdata, fit = object, re_formula = re_formula,
+    allow_new_levels = allow_new_levels, check_response = TRUE
+  )
   if (!is.null(standata$cens)) {
     warning2("Residuals may not be meaningful for censored models.")
   }
@@ -1908,7 +1935,7 @@ update.brmsfit <- function(object, formula., newdata = NULL, ...) {
   }
   object <- restructure(object)
   recompile <- FALSE
-  if (isTRUE(object$version < utils::packageVersion("brms"))) {
+  if (isTRUE(object$version$brms < utils::packageVersion("brms"))) {
     recompile <- TRUE
     warning2("Updating models fitted with older versions ", 
              "of brms may fail.")
@@ -1940,9 +1967,9 @@ update.brmsfit <- function(object, formula., newdata = NULL, ...) {
       ee_old <- parse_bf(formula(object))
       ee_new <- parse_bf(dots$formula)
       # no need to recompile the model when changing fixed effects only
-      dont_change <- c("random", "gam", "cs", "mo", "me")
-      n_old_fixef <- length(attr(terms(ee_old$fixed), "term.labels"))
-      n_new_fixef <- length(attr(terms(ee_new$fixed), "term.labels"))
+      dont_change <- c("re", "sm", "cs", "mo", "me")
+      n_old_fixef <- length(attr(terms(ee_old$auxpars$mu$fe), "term.labels"))
+      n_new_fixef <- length(attr(terms(ee_new$auxpars$mu$fe), "term.labels"))
       recompile <- recompile ||
         !is_equal(names(ee_old), names(ee_new)) ||
         !is_equal(ee_old[names(ee_old) %in% dont_change], 
@@ -2180,10 +2207,12 @@ log_lik.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
                             nsamples = NULL, pointwise = FALSE, ...) {
   contains_samples(object)
   object <- restructure(object)
-  draws <- extract_draws(x = object, newdata = newdata, 
-                         re_formula = re_formula, subset = subset,
-                         allow_new_levels = allow_new_levels,
-                         nsamples = nsamples, check_response = TRUE)
+  draws_args <- nlist(
+    x = object, newdata, re_formula, allow_new_levels,
+    subset, nsamples, check_response = TRUE
+  )
+  draws <- do.call(extract_draws, draws_args)
+  
   N <- if (!is.null(draws$data$N_tg)) draws$data$N_tg
        else if (is.cor_fixed(object$autocor)) 1
        else nrow(as.matrix(draws$data$Y))
@@ -2192,7 +2221,14 @@ log_lik.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
   if (pointwise) {
     loglik <- structure(loglik_fun, draws = draws, N = N)
   } else {
-    draws$eta <- get_eta(i = NULL, draws = draws)
+    if (is.list(draws$mu[["mv"]])) {
+      draws$mu <- get_eta(draws$mu)
+    }
+    for (ap in intersect(auxpars(), names(draws))) {
+      if (is.list(draws[[ap]])) {
+        draws[[ap]] <- get_auxpar(draws[[ap]])
+      }
+    }
     loglik <- do.call(cbind, lapply(seq_len(N), loglik_fun, draws = draws))
     old_order <- attr(draws$data, "old_order")
     # do not loglik reorder for ARMA covariance models
