@@ -18,6 +18,9 @@ update_data <- function(data, family, bterms,
   #   knots: a list of knot values for GAMMS
   # Returns:
   #   model.frame in long format with combined grouping variables if present
+  if (missing(data)) {
+    stop2("Argument 'data' is missing.")
+  }
   if (is.null(attr(data, "terms")) && "brms.frame" %in% class(data)) {
     # to avoid error described in #30
     # brms.frame class is deprecated as of brms > 0.7.0
@@ -55,7 +58,7 @@ update_data <- function(data, family, bterms,
       stop2("Variable names may not contain double underscores ",
             "or underscores at the end.")
     }
-    data <- combine_groups(data, get_random(bterms)$group, bterms$time$group)
+    data <- combine_groups(data, get_re(bterms)$group, bterms$time$group)
     data <- fix_factor_contrasts(data)
     attr(data, "knots") <- knots
     attr(data, "brmsframe") <- TRUE
@@ -149,17 +152,13 @@ data_rsv_intercept <- function(data, bterms) {
   # Args:
   #   data: data.frame or list
   #   bterms: object of class brmsterms
-  if (isTRUE(attr(bterms$fixed, "rsv_intercept"))) {
-    if (is.list(data) && length(data)) {
-      if ("intercept" %in% names(data)) {
-        stop2("Variable name 'intercept' is resevered in models ",
-              "without a population-level intercept.")
-      }
-      data$intercept <- rep(1, length(data[[1]]))
-    } else {
-      stop2("Argument 'data' must be a non empty data.frame ", 
-            "or list for this model.")
+  rsv_int <- ulapply(bterms$auxpars, function(x) attr(x$fe, "rsv_intercept"))
+  if (any(rsv_int)) {
+    if ("intercept" %in% names(data)) {
+      stop2("Variable name 'intercept' is resevered in models ",
+            "without a population-level intercept.")
     }
+    data$intercept <- rep(1, length(data[[1]]))
   }
   data
 }
@@ -234,14 +233,17 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
   if (!incl_autocor) {
     fit$autocor <- NULL
   }
-  if (is.null(newdata) || is(newdata, "standata")) {
+  if (is.null(newdata)) {
     # to shorten expressions in S3 methods such as predict.brmsfit
-    if (return_standata && is.null(newdata)) {
-      control <- list(not4stan = TRUE, save_order = TRUE)
+    if (return_standata) {
+      control <- nlist(not4stan = TRUE, save_order = TRUE,
+                       omit_response = !check_response)
       newdata <- standata(fit, re_formula = re_formula, control = control)
+    } else {
+      newdata <- model.frame(fit)
     }
     return(newdata)
-  } 
+  }
   newdata <- try(as.data.frame(newdata, silent = TRUE))
   if (is(newdata, "try-error")) {
     stop2("Argument 'newdata' must be coercible to a data.frame.")
@@ -252,7 +254,8 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
                      autocor = fit$autocor, resp_rhs_all = FALSE)
   resp_only_vars <- setdiff(all.vars(bterms$respform), 
                             all.vars(rhs(bterms$allvars)))
-  resp_only_vars <- c(resp_only_vars, all.vars(bterms[["dec"]]))  # fixes #162
+  # fixes #162
+  resp_only_vars <- c(resp_only_vars, all.vars(bterms$adforms$dec))
   missing_resp <- setdiff(resp_only_vars, names(newdata))
   if (check_response && length(missing_resp)) {
     stop2("Response variables must be specified in 'newdata'.\n",
@@ -263,12 +266,12 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
       newdata[[resp]] <- NA 
     }
   }
-  cens_vars <- all.vars(bterms$cens)
+  cens_vars <- all.vars(bterms$adforms$cens)
   for (v in setdiff(cens_vars, names(newdata))) {
     # censoring vars are unused in predict and related methods
     newdata[[v]] <- 0
   }
-  weights_vars <- all.vars(bterms$weights)
+  weights_vars <- all.vars(bterms$adforms$weights)
   for (v in setdiff(weights_vars, names(newdata))) {
     # weighting vars are unused in predict and related methods
     newdata[[v]] <- 1
@@ -348,9 +351,11 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
     }
     # brms:::update_data expects all original variables to be present
     # even if not actually used later on
-    rsv_vars <- rsv_vars(family(fit), nresp = length(bterms$response),
-                         rsv_intercept = has_rsv_intercept(bterms$formula),
-                         old_mv = attr(bterms$formula, "old_mv"))
+    rsv_vars <- rsv_vars(
+      family(fit), nresp = length(bterms$response),
+      rsv_intercept = has_rsv_intercept(bterms$formula),
+      old_mv = attr(bterms$formula, "old_mv")
+    )
     used_vars <- unique(c(names(newdata), all.vars(bterms$allvars), rsv_vars))
     all_vars <- all.vars(str2formula(names(model.frame(fit))))
     unused_vars <- setdiff(all_vars, used_vars)
@@ -375,10 +380,12 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
     }
   }
   if (return_standata) {
-    control <- list(is_newdata = TRUE, not4stan = TRUE, 
-                    old_levels = old_levels, save_order = TRUE, 
-                    omit_response = !check_response,
-                    old_cat <- is_old_categorical(fit))
+    control <- list(
+      is_newdata = TRUE, not4stan = TRUE, 
+      old_levels = old_levels, save_order = TRUE, 
+      omit_response = !check_response,
+      old_cat <- is_old_categorical(fit)
+    )
     old_terms <- attr(model.frame(fit), "terms")
     terms_attr <- c("variables", "predvars", "offset")
     control$terms_attr <- attributes(old_terms)[terms_attr]
@@ -399,9 +406,10 @@ amend_newdata <- function(newdata, fit, re_formula = NULL,
       fit$autocor$V <- diag(median_V, nrow(newdata))
     }
     knots <- attr(model.frame(fit), "knots")
-    newdata <- make_standata(new_formula, data = newdata, 
-                             family = fit$family, autocor = fit$autocor,
-                             knots = knots, control = control)
+    newdata <- make_standata(
+      new_formula, data = newdata, family = fit$family, 
+      autocor = fit$autocor, knots = knots, control = control
+    )
   }
   newdata
 }
@@ -487,7 +495,7 @@ make_smooth_list <- function(bterms, data) {
   # Returns:
   #   A named list of lists of smoothing objects
   #   one element per (non-linear) parameter    
-  if (has_splines(bterms)) {
+  if (has_smooths(bterms)) {
     knots <- attr(data, "knots")
     data <- rm_attr(data, "terms")
     gam_args <- list(data = data, knots = knots, 
@@ -495,26 +503,26 @@ make_smooth_list <- function(bterms, data) {
     if (length(bterms$nlpars)) {
       smooth <- named_list(names(bterms$nlpars))
       for (nlp in names(smooth)) {
-        splines <- get_spline_labels(bterms$nlpars[[nlp]])
-        for (i in seq_along(splines)) {
-          sc_args <- c(list(eval_spline(splines[i])), gam_args)
+        sm_labels <- get_sm_labels(bterms$nlpars[[nlp]])
+        for (i in seq_along(sm_labels)) {
+          sc_args <- c(list(eval_smooth(sm_labels[i])), gam_args)
           smooth[[nlp]][[i]] <- do.call(mgcv::smoothCon, sc_args)
         }
       }
     } else {
       smooth <- named_list("mu")
-      splines <- get_spline_labels(bterms)
-      for (i in seq_along(splines)) {
-        sc_args <- c(list(eval_spline(splines[i])), gam_args)
+      sm_labels <- get_sm_labels(bterms)
+      for (i in seq_along(sm_labels)) {
+        sc_args <- c(list(eval_smooth(sm_labels[i])), gam_args)
         smooth[["mu"]][[i]] <- do.call(mgcv::smoothCon, sc_args)
       }
     }
     auxpars <- names(bterms$auxpars)
     smooth <- c(smooth, named_list(auxpars))
     for (ap in auxpars) {
-      splines <- get_spline_labels(bterms$auxpars[[ap]])
-      for (i in seq_along(splines)) {
-        sc_args <- c(list(eval_spline(splines[i])), gam_args)
+      sm_labels <- get_sm_labels(bterms$auxpars[[ap]])
+      for (i in seq_along(sm_labels)) {
+        sc_args <- c(list(eval_smooth(sm_labels[i])), gam_args)
         smooth[[ap]][[i]] <- do.call(mgcv::smoothCon, sc_args)
       }
     }
@@ -554,285 +562,3 @@ arr_design_matrix <- function(Y, r, group)  {
   } 
   out
 }
-
-data_effects <- function(bterms, data, family = gaussian(),
-                         ranef = empty_ranef(), prior = brmsprior(), 
-                         autocor = cor_arma(), knots = NULL, nlpar = "", 
-                         not4stan = FALSE, smooth = NULL, Jmo = NULL) {
-  # combine data for all types of effects
-  # Args:
-  #   bterms: object of class brmsterms
-  #   data: the data passed by the user
-  #   family: the model family
-  #   prior: an object of class brmsprior
-  #   autocor: object of class 'cor_brms'
-  #   cov_ranef: name list of user-defined covariance matrices
-  #   knots: optional knot values for smoothing terms
-  #   nlpar: optional character string naming a non-linear parameter
-  #   not4stan: is the data for use in S3 methods only?
-  #   smooth: optional list of smoothing objects based on 
-  #           the original data
-  #   Jmo: optional precomputed values of Jmo for monotonic effects
-  # Returns:
-  #   A named list of data to be passed to Stan
-  data_fixef <- data_fixef(bterms, data = data, family = family, 
-                           autocor = autocor, nlpar = nlpar, 
-                           knots = knots, not4stan = not4stan,
-                           smooth = smooth)
-  data_monef <- data_monef(bterms, data = data, ranef = ranef,
-                           prior = prior, Jmo = Jmo, nlpar = nlpar)
-  data_ranef <- data_ranef(ranef, data = data, nlpar = nlpar, 
-                           not4stan = not4stan)
-  data_meef <- data_meef(bterms, data = data, nlpar = nlpar)
-  c(data_fixef, data_monef, data_ranef, data_meef)
-}
-
-data_fixef <- function(bterms, data, family = gaussian(),
-                       autocor = cor_arma(), knots = NULL,
-                       nlpar = "", not4stan = FALSE,
-                       smooth = NULL) {
-  # prepare data for fixed effects for use in Stan 
-  # Args: see data_effects
-  stopifnot(length(nlpar) == 1L)
-  out <- list()
-  p <- usc(nlpar, "prefix")
-  is_ordinal <- is_ordinal(family)
-  is_bsts <- is(autocor, "cor_bsts")
-  # the intercept is removed inside the Stan code for ordinal models
-  cols2remove <- if (is_ordinal && not4stan || is_bsts) "(Intercept)"
-  X <- get_model_matrix(rhs(bterms$fixed), data, 
-                        cols2remove = cols2remove)
-  splines <- get_spline_labels(bterms)
-  if (length(splines)) {
-    stopifnot(is.null(smooth) || length(smooth) == length(splines))
-    Xs <- Zs <- list()
-    new_smooths <- is.null(smooth)
-    if (new_smooths) {
-      smooth <- named_list(splines)
-      for (i in seq_along(splines)) {
-        smooth[[i]] <- mgcv::smoothCon(eval_spline(splines[i]), data = data, 
-                                       knots = knots, absorb.cons = TRUE)
-      }
-    }
-    by_levels <- named_list(splines)
-    ns <- 0
-    for (i in seq_along(smooth)) {
-      # may contain multiple terms when 'by' is a factor
-      for (j in seq_along(smooth[[i]])) {
-        ns <- ns + 1
-        sm <- smooth[[i]][[j]]
-        if (length(sm$by.level)) {
-          by_levels[[i]][j] <- sm$by.level
-        }
-        if (!new_smooths) {
-          sm$X <- mgcv::PredictMat(sm, rm_attr(data, "terms"))
-        }
-        rasm <- mgcv::smooth2random(sm, names(data))
-        Xs[[ns]] <- rasm$Xf
-        if (ncol(Xs[[ns]])) {
-          colnames(Xs[[ns]]) <- paste0(sm$label, "_", seq_len(ncol(Xs[[ns]])))
-        }
-        Zs <- lapply(rasm$rand, attr, "Xr")
-        Zs <- setNames(Zs, paste0("Zs", p, "_", ns, "_", seq_along(Zs)))
-        knots <- list(length(Zs), as.array(ulapply(Zs, ncol)))
-        knots <- setNames(knots, paste0(c("nb", "knots"), p, "_", ns))
-        out <- c(out, knots, Zs)
-      }
-    }
-    X <- cbind(X, do.call(cbind, Xs))
-    scols <- lapply(Xs, function(x) which(colnames(X) %in% colnames(x)))
-    X <- structure(X, smooth_cols = scols, by_levels = by_levels)
-    colnames(X) <- rename(colnames(X))
-  }
-  avoid_auxpars(colnames(X), bterms = bterms)
-  c(out, setNames(list(ncol(X), X), paste0(c("K", "X"), p)))
-}
-
-data_monef <- function(bterms, data, ranef = empty_ranef(),
-                       prior = brmsprior(), nlpar = "",
-                       Jmo = NULL) {
-  # prepare data for monotonic effects for use in Stan
-  # Args: see data_effects
-  stopifnot(length(nlpar) == 1L)
-  p <- if (nchar(nlpar)) paste0("_", nlpar) else ""
-  out <- list()
-  if (is.formula(bterms[["mo"]])) {
-    Xmo <- prepare_mo_vars(bterms$mo, data, check = is.null(Jmo))
-    avoid_auxpars(colnames(Xmo), bterms = bterms)
-    if (is.null(Jmo)) {
-      Jmo <- as.array(apply(Xmo, 2, max))
-    }
-    out <- c(out, setNames(list(ncol(Xmo), Xmo, Jmo), 
-                           paste0(c("Kmo", "Xmo", "Jmo"), p)))
-    # validate and assign vectors for dirichlet prior
-    monef <- colnames(Xmo)
-    for (i in seq_along(monef)) {
-      take <- prior$class == "simplex" & prior$coef == monef[i] & 
-              prior$nlpar == nlpar  
-      sprior <- prior$prior[take]
-      if (isTRUE(nchar(sprior) > 0L)) {
-        sprior <- eval2(sprior)
-        if (length(sprior) != Jmo[i]) {
-          stop2("Invalid dirichlet prior for the simplex of '", 
-                monef[i], "'. Expected input of length ", Jmo[i], 
-                " but found ", paste(sprior, collapse = ","))
-        }
-        out[[paste0("con_simplex", p, "_", i)]] <- sprior
-      } else {
-        out[[paste0("con_simplex", p, "_", i)]] <- rep(1, Jmo[i]) 
-      }
-    }
-  }
-  out
-}
-
-data_ranef <- function(ranef, data, nlpar = "", not4stan = FALSE) {
-  # prepare data for group-level effects for use in Stan
-  # Args: see data_effects
-  stopifnot(length(nlpar) == 1L)
-  out <- list()
-  take <- ranef$nlpar == nlpar & !ranef$type %in% c("mo", "me")
-  ranef <- ranef[take, ]
-  if (nrow(ranef)) {
-    Z <- lapply(ranef[!duplicated(ranef$gn), ]$form, 
-                get_model_matrix, data = data)
-    gn <- unique(ranef$gn)
-    for (i in seq_along(gn)) {
-      r <- ranef[ranef$gn == gn[i], ]
-      idp <- paste0(r$id[1], usc(nlpar, "prefix"))
-      if (isTRUE(not4stan)) {
-        # for internal use in S3 methods
-        if (ncol(Z[[i]]) == 1L) {
-          Z[[i]] <- as.vector(Z[[i]])
-        }
-        Zname <- paste0("Z_", gn[i])
-        out <- c(out, setNames(Z[i], Zname))
-      } else {
-        if (r$type[1] == "cs") {
-          ncatM1 <- nrow(r) / ncol(Z[[i]])
-          Z_temp <- vector("list", ncol(Z[[i]]))
-          for (k in seq_along(Z_temp)) {
-            Z_temp[[k]] <- replicate(ncatM1, Z[[i]][, k])
-          }
-          Z[[i]] <- do.call(cbind, Z_temp)
-        }
-        Zname <- paste0("Z_", idp, "_", r$cn)
-        for (j in seq_len(ncol(Z[[i]]))) {
-          out <- c(out, setNames(list(as.array(Z[[i]][, j])), Zname[j]))
-        }
-      }
-    }
-  }
-  out
-}
-
-data_group <- function(ranef, data, cov_ranef = NULL) {
-  # compute data specific for each group-level-ID
-  # Args:
-  #   ranef: data.frame returned by tidy_ranef
-  #   data: the model.frame
-  #   cov_ranef: name list of user-defined covariance matrices
-  out <- list()
-  ids <- unique(ranef$id)
-  for (id in ids) {
-    id_ranef <- ranef[ranef$id == id, ]
-    nranef <- nrow(id_ranef)
-    group <- id_ranef$group[1]
-    levels <- attr(ranef, "levels")[[group]]
-    if (id_ranef$gtype[1] == "mm") {
-      gs <- id_ranef$gcall[[1]]$groups
-      ngs <- length(gs)
-      weights <- id_ranef$gcall[[1]]$weights
-      if (is.formula(weights)) {
-        weights <- as.matrix(eval_rhs(weights, data))
-        if (!identical(dim(weights), c(nrow(data), ngs))) {
-          stop2("Grouping structure 'mm' expects 'weights' to be a matrix ", 
-                "with as many columns as grouping factors.")
-        }
-        weights <- sweep(weights, 1, rowSums(weights), "/")
-      } else {
-        # all members get equal weights by default
-        weights <- matrix(1 / ngs, nrow = nrow(data), ncol = ngs)
-      }
-      for (i in seq_along(gs)) {
-        temp <- list(as.array(match(get(gs[i], data), levels)), weights[, i])
-        out <- c(out, setNames(temp, paste0(c("J_", "W_"), id, "_", i)))
-      }
-    } else {
-      g <- id_ranef$gcall[[1]]$groups
-      out[[paste0("J_", id)]] <- as.array(match(get(g, data), levels))
-    }
-    temp <- list(length(levels), nranef, nranef * (nranef - 1) / 2)
-    out <- c(out, setNames(temp, paste0(c("N_", "M_", "NC_"), id)))
-    if (group %in% names(cov_ranef)) {
-      cov_mat <- as.matrix(cov_ranef[[group]])
-      if (!isSymmetric(unname(cov_mat))) {
-        stop2("Covariance matrix of grouping factor '", 
-              group, "' is not symmetric.")
-      }
-      found_levels <- rownames(cov_mat)
-      if (is.null(found_levels)) {
-        stop2("Row names are required for covariance matrix of '", group, "'.")
-      }
-      colnames(cov_mat) <- found_levels
-      found <- levels %in% found_levels
-      if (any(!found)) {
-        stop2("Row names of covariance matrix of '", group, 
-              "' do not match names of the grouping levels.")
-      }
-      cov_mat <- cov_mat[levels, levels, drop = FALSE]
-      evs <- eigen(cov_mat, symmetric = TRUE, only.values = TRUE)$values
-      if (min(evs) <= 0) {
-        stop2("Covariance matrix of grouping factor '", 
-              group, "' is not positive definite.")
-      }
-      out <- c(out, setNames(list(t(chol(cov_mat))), paste0("Lcov_", id)))
-    }
-  }
-  out
-}
-
-data_csef <- function(bterms, data) {
-  # prepare data for category specific effects for use in Stan
-  # Args:
-  #   bterms: object of class brmsterms
-  #   data: the data passed by the user
-  out <- list()
-  if (length(all_terms(bterms[["cs"]]))) {
-    Xcs <- get_model_matrix(bterms$cs, data)
-    avoid_auxpars(colnames(Xcs), bterms = bterms)
-    out <- c(out, list(Kcs = ncol(Xcs), Xcs = Xcs))
-  }
-  out
-}
-
-data_meef <- function(bterms, data, nlpar = "") {
-  # prepare data of meausurement error variables for use in Stan
-  # Args:
-  #   bterms: object of class brmsterms
-  #   data: the data passed by the user
-  out <- list()
-  meef <- get_me_labels(bterms, data)
-  if (length(meef)) {
-    p <- usc(nlpar, "prefix")
-    Cme <- get_model_matrix(bterms$me, data)
-    avoid_auxpars(colnames(Cme), bterms = bterms)
-    Cme <- Cme[, attr(meef, "not_one"), drop = FALSE]
-    Cme <- lapply(seq_len(ncol(Cme)), function(i) Cme[, i])
-    if (length(Cme)) {
-      Cme <- setNames(Cme, paste0("Cme", p, "_", seq_along(Cme)))
-    }
-    uni_me <- attr(meef, "uni_me")
-    Xn <- noise <- named_list(uni_me)
-    for (i in seq_along(uni_me)) {
-      temp <- eval2(uni_me[i], data)
-      Xn[[i]] <- as.array(attr(temp, "var"))
-      noise[[i]] <- as.array(attr(temp, "noise"))
-    }
-    names(Xn) <- paste0("Xn", p, "_", seq_along(Xn))
-    names(noise) <- paste0("noise", p, "_", seq_along(Xn))
-    Kme <- setNames(list(length(meef)), paste0("Kme", p))
-    out <- c(out, Xn, noise, Cme, Kme)
-  }
-  out
-}
