@@ -119,8 +119,11 @@
 #'   but this may result in too few shrinkage (Piironen & Vehtari, 2016).
 #'   It is thus possible to change the scale using argument \code{scale_global}
 #'   of the horseshoe prior, for instance \code{horseshoe(1, scale_global = 0.5)}.
-#'   For recommendations how to properly set the global scale see 
-#'   Piironen and Vehtari (2016).
+#'   In linear models, \code{scale_global} will internally be multiplied by the 
+#'   residual standard deviation parameter \code{sigma}. See Piironen and 
+#'   Vehtari (2016) for recommendations how to properly set the global scale.
+#'   The degrees of freedom of the global shrinkage prior may also be 
+#'   adjusted via argument \code{df_global}. 
 #'   To make sure that shrinkage can equally affect all coefficients, 
 #'   predictors should be one the same scale. 
 #'   Generally, models with horseshoe priors a more likely than other models
@@ -132,7 +135,7 @@
 #'   Another shrinkage prior is the so-called \emph{lasso} prior.
 #'   It is the Bayesian equivalent to the LASSO method for performing
 #'   variable selection (Park & Casella, 2008).
-#'   With this prior, independent Laplace (i.e., double exponential) priors 
+#'   With this prior, independent Laplace (i.e. double exponential) priors 
 #'   are placed on the population-level effects. 
 #'   The scale of the Laplace priors depends on a tuning parameter
 #'   that controls the amount of shrinkage. In \pkg{brms}, the inverse
@@ -515,6 +518,7 @@ get_prior <- function(formula, data, family = NULL,
   autocor <- check_autocor(autocor)
   bterms <- parse_bf(formula, family = family)
   data <- update_data(data, family = family, bterms = bterms)
+  ranef <- tidy_ranef(bterms, data)
   
   # ensure that RE and residual SDs only have a weakly informative prior by default
   Y <- unname(model.response(data))
@@ -533,50 +537,43 @@ get_prior <- function(formula, data, family = NULL,
   
   # initialize output
   prior <- empty_brmsprior()
-  # priors for primary regression effects
-  if (length(bterms$nlpars)) {
-    nlpars <- names(bterms$nlpars)
-    for (i in seq_along(nlpars)) {
-      prior_eff <- get_prior_effects(bterms$nlpars[[i]], data = data, 
-                                     autocor = autocor, nlpar = nlpars[i],
-                                     spec_intercept = FALSE,
-                                     def_scale_prior = def_scale_prior,
-                                     internal = internal)
-      prior <- rbind(prior, prior_eff)
-    }
-  } else {
-    if (length(bterms$response) > 1L) {
-      # priors for effects in multivariate models
-      for (r in c("", bterms$response)) {
-        # r = "" adds global priors affecting parameters of all responses
-        prior_eff <- get_prior_effects(bterms, data = data, autocor = autocor,
-                                       def_scale_prior = def_scale_prior,
-                                       internal = internal, nlpar = r)
-        prior <- rbind(prior, prior_eff)
-      }
-    } else {
-      # priors for effects in univariate models
-      prior_eff <- get_prior_effects(bterms, data = data, autocor = autocor,
-                                     def_scale_prior = def_scale_prior,
-                                     internal = internal)
+  if (length(bterms$response) > 1L) {
+    # priors for effects in multivariate models
+    for (r in c("", bterms$response)) {
+      # r = "" adds global priors affecting responses
+      prior_eff <- prior_effects(
+        bterms$auxpars[["mu"]], data = data,
+        def_scale_prior = def_scale_prior,
+        nlpar = r, internal = internal
+      )
       prior <- rbind(prior, prior_eff)
     }
+    bterms$auxpars[["mu"]] <- NULL
   }
   # priors for auxiliary parameters
-  def_auxprior <- c(sigma = def_scale_prior, shape = "gamma(0.01, 0.01)",
-                    nu = "gamma(2, 0.1)", phi = "gamma(0.01, 0.01)",
-                    kappa = "gamma(2, 0.01)", beta = "gamma(1, 0.1)", 
-                    zi = "beta(1, 1)", hu = "beta(1, 1)", 
-                    bs = "gamma(1, 1)", ndt = "uniform(0, min_Y)", 
-                    bias = "beta(1, 1)", disc = NA)
-  valid_auxpars <- valid_auxpars(family, bterms = bterms, autocor = autocor)
+  def_auxprior <- c(
+    sigma = def_scale_prior, 
+    shape = "gamma(0.01, 0.01)",
+    nu = "gamma(2, 0.1)", 
+    phi = "gamma(0.01, 0.01)",
+    kappa = "gamma(2, 0.01)", 
+    beta = "gamma(1, 0.1)", 
+    zi = "beta(1, 1)", 
+    hu = "beta(1, 1)", 
+    bs = "gamma(1, 1)", 
+    ndt = "uniform(0, min_Y)", 
+    bias = "beta(1, 1)", 
+    xi = "normal(0, 2.5)",
+    disc = NA,
+    mu = NA
+  )
+  valid_auxpars <- valid_auxpars(family, bterms = bterms)
   for (ap in valid_auxpars) {
     if (!is.null(bterms$auxpars[[ap]])) {
-      auxprior <- get_prior_effects(bterms$auxpars[[ap]], data = data,
-                                    autocor = autocor, nlpar = ap,
-                                    spec_intercept = FALSE,
-                                    def_scale_prior = def_scale_prior,
-                                    internal = internal)
+      auxprior <- prior_effects(
+        bterms$auxpars[[ap]], data = data, nlpar = ap,
+        def_scale_prior = def_scale_prior
+      )
     } else if (!is.na(def_auxprior[ap])) {
       auxprior <- brmsprior(class = ap, prior = def_auxprior[ap])
     } else {
@@ -585,12 +582,12 @@ get_prior <- function(formula, data, family = NULL,
     prior <- rbind(prior, auxprior)
   }
   # priors of group-level parameters
-  ranef <- tidy_ranef(bterms, data)
-  prior_ranef <- get_prior_ranef(ranef, def_scale_prior = def_scale_prior,
-                                 global_sd = length(bterms$response) > 1L,
-                                 internal = internal)
-  prior <- rbind(prior, prior_ranef)
-  
+  prior_re <- prior_re(
+    ranef, def_scale_prior = def_scale_prior,
+    global_sd = length(bterms$response) > 1L,
+    internal = internal
+  )
+  prior <- rbind(prior, prior_re)
   # prior for the delta parameter for equidistant thresholds
   if (is_ordinal(family) && threshold == "equidistant") {
     prior <- rbind(prior, brmsprior(class = "delta"))
@@ -599,12 +596,14 @@ get_prior <- function(formula, data, family = NULL,
   if (is_linear(family) && length(bterms$response) > 1L) {
     sigma_coef <- c("", bterms$response)
     sigma_prior <- c(def_scale_prior, rep("", length(bterms$response)))
-    sigma_prior <- brmsprior(class = "sigma", coef = sigma_coef,
-                             prior = sigma_prior)
+    sigma_prior <- brmsprior(
+      class = "sigma", coef = sigma_coef, prior = sigma_prior
+    )
     prior <- rbind(prior, sigma_prior)
     if (internal) {
-      prior <- rbind(prior, brmsprior(class = "Lrescor", 
-                                      prior = "lkj_corr_cholesky(1)"))
+      prior <- rbind(prior, 
+        brmsprior(class = "Lrescor", prior = "lkj_corr_cholesky(1)")
+      )
     } else {
       prior <- rbind(prior, brmsprior(class = "rescor", prior = "lkj(1)"))
     }
@@ -621,68 +620,80 @@ get_prior <- function(formula, data, family = NULL,
     prior <- rbind(prior, brmsprior(class = "arr"))
   }
   if (is(autocor, "cor_bsts")) {
-    prior <- rbind(prior, brmsprior(class = "sigmaLL", 
-                                    prior = def_scale_prior))
+    prior <- rbind(prior, 
+      brmsprior(class = "sigmaLL", prior = def_scale_prior)
+    )
   }
   # do not remove unique(.)
   prior <- unique(prior[with(prior, order(nlpar, class, group, coef)), ])
-  rownames(prior) <- seq_len(nrow(prior))
+  rownames(prior) <- NULL
   structure(prior, class = c("brmsprior", "data.frame"))
 }
 
-get_prior_effects <- function(bterms, data, autocor = cor_arma(), 
-                              nlpar = "", spec_intercept = TRUE,
-                              def_scale_prior = "", internal = FALSE) {
-  # wrapper function to get priors for various kinds of effects
-  # don't use the family argument here to avoid
-  # removal of the intercept for ordinal models
-  # group-level priors are prepared separately
-  # Args:
-  #   spec_intercept: special parameter class for the FE Intercept? 
-  fixef <- colnames(data_fixef(bterms, data, autocor = autocor)$X)
-  spec_intercept <- has_intercept(bterms$fixed) && spec_intercept
-  prior_fixef <- get_prior_fixef(fixef, spec_intercept = spec_intercept,
-                                 nlpar = nlpar, internal = internal)
-  monef <- all_terms(bterms$mo)
-  prior_monef <- get_prior_monef(monef, fixef = fixef, nlpar = nlpar)
-  splines <- get_spline_labels(bterms)
-  prior_splines <- get_prior_splines(splines, def_scale_prior, nlpar = nlpar)
-  csef <- colnames(get_model_matrix(bterms$cs, data = data))
-  prior_csef <- get_prior_csef(csef, fixef = fixef)
-  prior_meef <- get_prior_meef(get_me_labels(bterms, data))
-  rbind(prior_fixef, prior_monef, prior_splines, prior_csef, prior_meef)
+#' @export
+prior_effects.btl <- function(x, data, nlpar = "", spec_intercept = TRUE,
+                              def_scale_prior = "", ...) {
+  # TODO: doc
+  nlpar <- check_nlpar(nlpar)
+  fixef <- colnames(data_fe(x, data)$X)
+  spec_intercept <- has_intercept(x$fe) && spec_intercept
+  prior_fe <- prior_fe(
+    fixef, spec_intercept = spec_intercept, nlpar = nlpar
+  )
+  monef <- all_terms(x$mo)
+  prior_mo <- prior_mo(monef, fixef = fixef, nlpar = nlpar)
+  smooths <- get_sm_labels(x)
+  prior_sm <- prior_sm(smooths, def_scale_prior, nlpar = nlpar)
+  csef <- colnames(get_model_matrix(x$cs, data = data))
+  prior_cs <- prior_cs(csef, fixef = fixef)
+  prior_me <- prior_me(get_me_labels(x, data))
+  rbind(prior_fe, prior_mo, prior_sm, prior_cs, prior_me)
 }
 
-get_prior_fixef <- function(fixef, spec_intercept = TRUE, 
-                            nlpar = "", internal = FALSE) {
-  # priors for fixed effects parameters
-  # Args:
-  #   fixef: names of the fixed effects
-  #   spec_intercept: special parameter class for the Intercept? 
-  #   internal: see get_prior
-  # Returns:
-  #   an object of class brmsprior
+#' @export
+prior_effects.btnl <- function(x, data, def_scale_prior = "", ...) {
+  # TODO: doc
+  nlpars <- names(x$nlpars)
   prior <- empty_brmsprior()
-  if (length(fixef)) {
-    prior <- rbind(prior, brmsprior(class = "b", coef = c("", fixef),
-                                    nlpar = nlpar)) 
-  }
-  if (spec_intercept) {
-    prior <- rbind(prior, brmsprior(class = "Intercept", coef = "",
-                                    nlpar = nlpar))
-    if (internal) {
-      prior <- rbind(prior, brmsprior(class = "temp", coef = "Intercept",
-                                      nlpar = nlpar))
-    }
+  for (i in seq_along(nlpars)) {
+    prior_eff <- prior_effects(
+      x$nlpars[[i]], data = data, nlpar = nlpars[i], 
+      def_scale_prior = def_scale_prior,
+      spec_intercept = FALSE
+    )
+    prior <- rbind(prior, prior_eff)
   }
   prior
 }
 
-get_prior_monef <- function(monef, fixef = NULL, nlpar = "") {
+prior_fe <- function(fixef, spec_intercept = TRUE, nlpar = "") {
+  # priors for population-level parameters
+  # Args:
+  #   fixef: names of the population-level effects
+  #   spec_intercept: special parameter class for the Intercept? 
+  # Returns:
+  #   an object of class brmsprior
+  prior <- empty_brmsprior()
+  if (spec_intercept) {
+    prior <- rbind(prior, 
+      brmsprior(class = "Intercept", coef = "", nlpar = nlpar),
+      brmsprior(class = "b", coef = "Intercept", nlpar = nlpar)
+    )
+    fixef <- setdiff(fixef, "Intercept")
+  }
+  if (length(fixef)) {
+    prior <- rbind(prior, 
+      brmsprior(class = "b", coef = c("", fixef), nlpar = nlpar)
+    ) 
+  }
+  prior
+}
+
+prior_mo <- function(monef, fixef = NULL, nlpar = "") {
   # priors for monotonic effects parameters
   # Args:
   #   monef: names of the monotonic effects
-  #   fixef: names of the fixed effects
+  #   fixef: names of the population-level effects
   #   nlpar: optional name of a non-linear parameter
   # Returns:
   #   an object of class brmsprior
@@ -690,54 +701,55 @@ get_prior_monef <- function(monef, fixef = NULL, nlpar = "") {
   if (length(monef)) {
     invalid <- intersect(fixef, monef)
     if (length(invalid)) {
-      stop(paste("Variables cannot be modeled as fixed and", 
-                 "monotonic effects at the same time.", 
-                 "\nError occured for variables:", 
-                 paste(invalid, collapse = ", ")), call. = FALSE)
+      stop2("Variables cannot be modeled as fixed and ", 
+            "monotonic effects at the same time. ", 
+            "\nError occured for variables: ", 
+            collapse_comma(invalid))
     }
-    prior <- rbind(brmsprior(class = "b", coef = c("", monef), 
-                             nlpar = nlpar),
-                   brmsprior(class = "simplex", coef = monef, 
-                             nlpar = nlpar))
+    prior <- rbind(
+      brmsprior(class = "b", coef = c("", monef), nlpar = nlpar),
+      brmsprior(class = "simplex", coef = monef, nlpar = nlpar)
+    )
   }
   prior
 }
 
-get_prior_csef <- function(csef, fixef = NULL) {
+prior_cs <- function(csef, fixef = NULL) {
   # priors for category spcific effects parameters
   # Args:
   #   csef: names of the category specific effects
-  #   fixef: names of the fixed effects
+  #   fixef: names of the population-level effects
   # Returns:
   #   an object of class brmsprior
   prior <- empty_brmsprior()
   if (length(csef)) {
     invalid <- intersect(fixef, csef)
     if (length(invalid)) {
-      stop(paste("Variables cannot be modeled as fixed and", 
-                 "category specific effects at the same time.", 
-                 "\nError occured for variables:", 
-                 paste(invalid, collapse = ", ")), call. = FALSE)
+      stop2("Variables cannot be modeled as fixed and ", 
+            "category specific effects at the same time. ", 
+            "\nError occured for variables: ", 
+            collapse_comma(invalid))
     }
     prior <- brmsprior(class = "b", coef = c("", csef))
   }
   prior
 }
 
-get_prior_meef <- function(meef, nlpar = "") {
+prior_me <- function(meef, nlpar = "") {
   # default priors of coefficients of noisy terms
   # Args:
   #   meef: terms containing noisy variables
   prior <- empty_brmsprior()
   if (length(meef)) {
-    prior <- brmsprior(class = "b", coef = c("", rename(meef)),
-                       nlpar = nlpar)
+    prior <- brmsprior(
+      class = "b", coef = c("", rename(meef)), nlpar = nlpar
+    )
   }
   prior
 }
 
-get_prior_ranef <- function(ranef, def_scale_prior, 
-                            global_sd = FALSE, internal = FALSE) {
+prior_re <- function(ranef, def_scale_prior, global_sd = FALSE,
+                     internal = FALSE) {
   # priors for random effects parameters
   # Args:
   #   ranef: a list returned by tidy_ranef
@@ -755,13 +767,12 @@ get_prior_ranef <- function(ranef, def_scale_prior,
     if (global_sd) {
       global_sd_prior <- rep("", length(setdiff(nlpars, "")))
       global_sd_prior <- c(def_scale_prior, global_sd_prior)
-      global_sd_prior <- brmsprior(class = "sd", 
-                                   prior = global_sd_prior,
-                                   nlpar = union("", nlpars))
+      global_sd_prior <- brmsprior(
+        class = "sd", prior = global_sd_prior, nlpar = union("", nlpars)
+      )
     } else {
-      global_sd_prior <- brmsprior(class = "sd", 
-                                   prior = def_scale_prior,
-                                   nlpar = nlpars)
+      global_sd_prior <- 
+        brmsprior(class = "sd", prior = def_scale_prior, nlpar = nlpars)
     }
     prior <- rbind(prior, global_sd_prior)
     for (id in unique(ranef$id)) {
@@ -772,24 +783,26 @@ get_prior_ranef <- function(ranef, def_scale_prior,
         brmsprior(class = "sd", group = group, 
                   nlpar = unique(r$nlpar)),
         brmsprior(class = "sd", coef = r$coef, 
-                  group = group, nlpar = r$nlpar))
+                  group = group, nlpar = r$nlpar)
+      )
       # detect duplicated group-level effects
       J <- with(prior, class == "sd" & nchar(coef))
       dupli <- duplicated(prior[J, ])
       if (any(dupli)) {
-        stop("Duplicated group-level effects detected for group ", 
-             group, call. = FALSE)
+        stop2("Duplicated group-level effects detected for group ", group)
       }
       # include correlation parameters
       if (isTRUE(r$cor[1]) && nrow(r) > 1L) {
         if (internal) {
           prior <- rbind(prior, 
             brmsprior(class = "L", group = c("", group),
-                      prior = c("lkj_corr_cholesky(1)", "")))
+                      prior = c("lkj_corr_cholesky(1)", ""))
+          )
         } else {
           prior <- rbind(prior, 
             brmsprior(class = "cor", group = c("", group),
-                      prior = c("lkj(1)", "")))
+                      prior = c("lkj(1)", ""))
+          )
         }
       }
     }
@@ -797,16 +810,16 @@ get_prior_ranef <- function(ranef, def_scale_prior,
   prior
 }
 
-get_prior_splines <- function(splines, def_scale_prior, nlpar = "") {
-  # priors for GAMM models
+prior_sm <- function(smooths, def_scale_prior, nlpar = "") {
+  # priors for smooth terms
   # Args:
-  #   splines: names of the spline terms
+  #   smooths: names of the smooth terms
   #   def_scale_prior: a character string defining the default
-  #                    prior for spline SDs
+  #                    prior for smooth SDs
   #   nlpar: optional name of a non-linear parameter
-  if (length(splines)) {
-    prior_strings <- c(def_scale_prior, rep("", length(splines)))
-    prior <- brmsprior(class = "sds", coef = c("", splines), 
+  if (length(smooths)) {
+    prior_strings <- c(def_scale_prior, rep("", length(smooths)))
+    prior <- brmsprior(class = "sds", coef = c("", smooths), 
                        prior = prior_strings, nlpar = nlpar)
   } else {
     prior <- empty_brmsprior()
@@ -814,7 +827,7 @@ get_prior_splines <- function(splines, def_scale_prior, nlpar = "") {
   prior
 }
 
-check_prior <- function(prior, formula, data = NULL, family = gaussian(), 
+check_prior <- function(prior, formula, data = NULL, family = NULL, 
                         sample_prior = FALSE, autocor = NULL, 
                         threshold = "flexible", check_rows = NULL, 
                         warn = FALSE) {
@@ -834,9 +847,10 @@ check_prior <- function(prior, formula, data = NULL, family = gaussian(),
   }
   formula <- bf(formula)
   bterms <- parse_bf(formula, family = family)  
-  all_priors <- get_prior(formula = formula, data = data, 
-                          family = family, autocor = autocor, 
-                          threshold = threshold, internal = TRUE)
+  all_priors <- get_prior(
+    formula = formula, data = data, family = family, 
+    autocor = autocor, threshold = threshold, internal = TRUE
+  )
   if (is.null(prior)) {
     prior <- all_priors  
   }
@@ -845,15 +859,16 @@ check_prior <- function(prior, formula, data = NULL, family = gaussian(),
   prior_no_checks <- prior[no_checks, ]
   prior <- prior[!no_checks, ]
   # check for duplicated priors
-  prior$class <- rename(prior$class, symbols = c("^cor$", "^rescor$"), 
-                        subs = c("L", "Lrescor"), fixed = FALSE)
+  prior$class <- rename(
+    prior$class, symbols = c("^cor$", "^rescor$"), 
+    subs = c("L", "Lrescor"), fixed = FALSE
+  )
   duplicated_input <- duplicated(prior[, 2:5])
   if (any(duplicated_input)) {
     stop2("Duplicated prior specifications are not allowed.")
   }
   # handle special priors that are not explictly coded as functions in Stan
-  has_specef <- is.formula(bterms[["mo"]]) || is.formula(bterms[["cs"]])
-  prior <- handle_special_priors(prior, has_specef = has_specef)  
+  prior <- handle_special_priors(prior, bterms)  
   # check if parameters in prior are valid
   if (nrow(prior)) {
     valid <- which(duplicated(rbind(all_priors[, 2:5], prior[, 2:5])))
@@ -877,44 +892,48 @@ check_prior <- function(prior, formula, data = NULL, family = gaussian(),
     int_prior <- prior[int_index, ]
     bint_index <- which(prior$class == "b" & prior$coef %in% "Intercept")
     bint_prior <- prior[bint_index, ]
-    for (t in which(prior$class %in% "temp" & prior$coef %in% "Intercept")) {
-      ti <- match(prior$nlpar[t], int_prior$nlpar)
+    for (t in int_index) {
       tb <- match(prior$nlpar[t], bint_prior$nlpar) 
-      if (!is.na(ti) && nzchar(int_prior$prior[ti])) {
-        # take 'Intercept' priors first if specified
-        prior$prior[t] <- int_prior$prior[ti]
-      } else if (!is.na(tb) && nzchar(bint_prior$prior[tb])) {
-        # fall back to 'b' (fixed effects) priors
-        prior$prior[t] <- bint_prior$prior[tb]
+      if (!is.na(tb) && nzchar(bint_prior$prior[tb])) {
+        # fall back to 'b' priors deprecated as of brms 1.5.0
+        if (nzchar(prior$prior[t])) {
+          stop2("Duplicated prior definitions detected ", 
+                "for the population-level intercept.")
+        } else {
+          warning2(
+            "Setting a prior on the population-level intercept",
+            "\nvia (class = 'b', coef = 'Intercept') is deprecated.",
+            "\nPlease use (class = 'Intercept', coef = '') instead."
+          )
+          prior$prior[t] <- bint_prior$prior[tb]
+        }
       }
     }
-    rows2remove <- c(rows2remove, int_index, bint_index)
+    rows2remove <- c(rows2remove, bint_index)
   }
   # prepare priors of monotonic effects
   mo_forms <- get_effect(bterms, "mo")
   for (k in seq_along(mo_forms)) {
     monef <- colnames(get_model_matrix(mo_forms[[k]], data = data))
     for (i in seq_along(monef)) {
-      take <- with(prior, class == "simplex" & coef == monef[i] &
-                          nlpar == names(mo_forms)[k])
+      take <- with(prior, 
+        class == "simplex" & coef == monef[i] & nlpar == names(mo_forms)[k]
+      )
       simplex_prior <- paste0(".", prior$prior[take])
       if (nchar(simplex_prior) > 1L) {
-        simplex_prior <- paste(eval(parse(text = simplex_prior)),
-                               collapse = ",")
+        simplex_prior <- paste(eval2(simplex_prior), collapse = ",")
         prior$prior[take] <- paste0("dirichlet(c(", simplex_prior, "))")
       }
     }
   }
   # check if priors for non-linear parameters are defined
-  if (length(bterms$nlpars)) {
-    nlpars <- names(bterms$nlpars)
-    for (nlp in nlpars) {
-      nlp_prior <- prior$prior[with(prior, nlpar == nlp & class == "b")]
-      if (!any(as.logical(nchar(nlp_prior)))) {
-        stop2("Priors on population-level effects are required in ",
-              "non-linear models,\nbut none were found for parameter ", 
-              "'", nlp, "'. \nSee help(set_prior) for more details.")
-      }
+  nlpars <- names(bterms$auxpars$mu$nlpars)
+  for (nlp in nlpars) {
+    nlp_prior <- prior$prior[with(prior, nlpar == nlp & class == "b")]
+    if (!any(as.logical(nchar(nlp_prior)))) {
+      stop2("Priors on population-level effects are required in ",
+            "non-linear models,\nbut none were found for parameter ", 
+            "'", nlp, "'. \nSee help(set_prior) for more details.")
     }
   }
   if (length(rows2remove)) {   
@@ -922,7 +941,7 @@ check_prior <- function(prior, formula, data = NULL, family = gaussian(),
   }
   prior <- prior[with(prior, order(nlpar, class, group, coef)), ]
   prior <- rbind(prior, prior_no_checks)
-  rownames(prior) <- seq_len(nrow(prior))
+  rownames(prior) <- NULL
   attr(prior, "prior_only") <- prior_only
   attr(prior, "checked") <- TRUE
   prior
@@ -940,10 +959,12 @@ check_prior_content <- function(prior, family = gaussian(), warn = TRUE) {
   stopifnot(is.family(family))
   family <- family$family
   if (nrow(prior)) {
-    lb_priors <- c("lognormal", "chi_square", "inv_chi_square",
-                   "scaled_inv_chi_square", "exponential", "gamma",
-                   "inv_gamma", "weibull", "frechet", "rayleigh",
-                   "pareto", "pareto_type_2")
+    lb_priors <- c(
+      "lognormal", "chi_square", "inv_chi_square",
+      "scaled_inv_chi_square", "exponential", "gamma",
+      "inv_gamma", "weibull", "frechet", "rayleigh",
+      "pareto", "pareto_type_2"
+    )
     lb_priors_reg <- paste0("^(", paste0(lb_priors, collapse = "|"), ")")
     ulb_priors <- c("beta", "uniform", "von_mises")
     ulb_priors_reg <- paste0("^(", paste0(ulb_priors, collapse = "|"), ")")
@@ -959,8 +980,10 @@ check_prior_content <- function(prior, family = gaussian(), warn = TRUE) {
       has_lb_prior <- grepl(lb_priors_reg, prior$prior[i])
       has_ulb_prior <- grepl(ulb_priors_reg, prior$prior[i])
       # priors with nchar(coef) inherit their boundaries 
-      j <- with(prior, which(class == class[i] & group == group[i] & 
-                               nlpar == nlpar[i] & !nchar(coef)))
+      j <- with(prior, 
+        which(class == class[i] & group == group[i] & 
+              nlpar == nlpar[i] & !nchar(coef))
+      )
       bound <- if (length(j)) prior$bound[j] else ""
       has_lb <- grepl("lower", bound)
       has_ub <- grepl("upper", bound)
@@ -977,9 +1000,9 @@ check_prior_content <- function(prior, family = gaussian(), warn = TRUE) {
         }
       } else if (prior$class[i] %in% cor_pars) {
         if (nchar(prior$prior[i]) && !grepl("^lkj", prior$prior[i])) {
-          stop(paste("Currently 'lkj' is the only valid prior",
-                     "for group-level correlations. See help(set_prior)",
-                     "for more details."), call. = FALSE)
+          stop2("Currently 'lkj' is the only valid prior ",
+                "for group-level correlations. See help(set_prior) ",
+                "for more details.")
         }
       } else if (prior$class[i] %in% autocor_pars) {
         if (prior$bound[i] != "<lower=-1,upper=1>") {
@@ -987,9 +1010,9 @@ check_prior_content <- function(prior, family = gaussian(), warn = TRUE) {
         } 
       } else if (prior$class[i] == "simplex") {
         if (nchar(prior$prior[i]) && !grepl("^dirichlet\\(", prior$prior[i])) {
-          stop(paste("Currently 'dirichlet' is the only valid prior",
-                     "for simplex parameters. See help(set_prior)",
-                     "for more details."), call. = FALSE)
+          stop2("Currently 'dirichlet' is the only valid prior ",
+                "for simplex parameters. See help(set_prior) ",
+                "for more details.")
         }
       }
     }  # end for  
@@ -1015,7 +1038,7 @@ check_prior_content <- function(prior, family = gaussian(), warn = TRUE) {
   invisible(NULL)
 }
 
-handle_special_priors <- function(prior, has_specef = FALSE) {
+handle_special_priors <- function(prior, bterms) {
   # look for special priors such as horseshoe and process them appropriately
   # Args:
   #   prior: an object of class brmsprior
@@ -1027,17 +1050,22 @@ handle_special_priors <- function(prior, has_specef = FALSE) {
   if (length(b_index)) {
     b_prior <- prior$prior[b_index]
     if (any(grepl("^(horseshoe|lasso)\\(", b_prior))) {
-      # horseshoe prior for fixed effects parameters
+      # horseshoe prior for population-level parameters
       if (any(nchar(prior$nlpar))) {
         stop2("Horseshoe or lasso priors are not yet allowed ", 
-              "in non-linear models.")
+              "in non-linear or distributional models.")
       }
-      if (has_specef) {
-        stop2("Horseshoe or lasso priors are not yet allowed ", 
-              "in models with monotonic or category specific effects.")
+      if (any(nzchar(prior[b_index, "bound"]))) {
+        stop2("Boundaries for population-level effects are not", 
+              "allowed when using the horseshoe or lasso priors.")
       }
-      b_coef_indices <- which(prior$class == "b" & nchar(prior$coef) &
-                              prior$coef != "Intercept")
+      if (any(ulapply(bterms$auxpars$mu[c("me", "mo", "cs")], is.formula))) {
+        stop2("Horseshoe or lasso priors are not yet allowed ",
+              "in models with special population-level effects.")
+      }
+      b_coef_indices <- which(
+        prior$class == "b" & nchar(prior$coef) & prior$coef != "Intercept"
+      )
       if (any(nchar(prior$prior[b_coef_indices]))) {
         stop2("Defining priors for single population-level parameters",
               "is not allowed when using horseshoe or lasso priors",
@@ -1045,17 +1073,29 @@ handle_special_priors <- function(prior, has_specef = FALSE) {
       }
       if (grepl("^horseshoe\\(", b_prior)) {
         hs <- eval2(b_prior)
-        prior_attr[c("hs_df", "hs_scale_global")] <- hs[c("df", "scale_global")]
-        prior$prior[b_index] <- hs$prior
+        prior$prior[b_index] <- ""
+        prior_attr$hs_df <- attr(hs, "df")
+        prior_attr$hs_df_global <- attr(hs, "df_global")
+        scale_global <- attr(hs, "scale_global")
+        has_sigma <- has_sigma(bterms$family, bterms)
+        if (has_sigma && !is.formula(bterms$auxpars$sigma)) {
+          scale_global <- paste(scale_global, "* sigma")
+        }
+        prior_attr$hs_scale_global <- scale_global
       } else if (grepl("^lasso\\(", b_prior)) {
         lasso <- eval2(b_prior)
-        prior_attr[c("lasso_df", "lasso_scale")] <- lasso[c("df", "scale")]
-        prior$prior[b_index] <- lasso$prior
+        lasso_scale <- paste(attr(lasso, "scale"), "* lasso_inv_lambda")
+        lasso_prior <- paste0("double_exponential(0, ", lasso_scale, ")")
+        prior$prior[b_index] <- lasso_prior
+        prior_attr$lasso_df <- attr(lasso, "df")
+        prior_attr$lasso_scale <- attr(lasso, "scale")
       }
     }
   }
   # expand lkj correlation prior to full name
-  prior$prior <- sub("^(lkj\\(|lkj_corr\\()", "lkj_corr_cholesky(", prior$prior)
+  prior$prior <- sub(
+    "^(lkj\\(|lkj_corr\\()", "lkj_corr_cholesky(", prior$prior
+  )
   do.call(structure, c(list(prior), prior_attr))
 }
 
@@ -1157,30 +1197,80 @@ dirichlet <- function(...) {
   out
 }
 
-horseshoe <- function(df = 1, scale_global = 1) {
-  # validate input for the horseshoe prior 
-  # Args:
-  #   df: degrees of freedom of the local parameters
-  #   scale_global: scale of the global cauchy prior
+
+#' Set up a horseshoe prior in \pkg{brms}
+#' 
+#' Function used to set up a horseshoe prior for population-level effects 
+#' in \pkg{brms}. The function does not evaluate its arguments --
+#' it exists purely to help set up the model.
+#' 
+#' @param df Degrees of freedom of student-t prior of the 
+#'   local shrinkage parameters. Defaults to \code{1}.
+#' @param scale_global Scale of the student-t prior of the global shrinkage 
+#'   parameter. Defaults to \code{1}. 
+#'   In linear models, \code{scale_global} will internally be 
+#'   multiplied by the residual standard deviation parameter \code{sigma}.
+#' @param df_global Degrees of freedom of student-t prior of the 
+#'   global shrinkage parameter. Defaults to \code{1}.
+#'   
+#' @return A character string obtained by \code{match.call()} with
+#'   additional arguments.
+#'   
+#' @details See \code{\link[brms:set_prior]{set_prior}} for more details
+#'   about the horseshoe prior.
+#'   
+#' @seealso \code{\link[brms:set_prior]{set_prior}}
+#'   
+#' @examples 
+#' set_prior(horseshoe(df = 3, scale_global = 2))
+#' 
+#' @export
+horseshoe <- function(df = 1, scale_global = 1, df_global = 1) {
+  out <- deparse(match.call())
   df <- round(as.numeric(df)[1], 5)
+  df_global <- round(as.numeric(df_global)[1], 5)
   scale_global <- round(as.numeric(scale_global)[1], 5)
   if (!isTRUE(df > 0)) {
     stop2("Invalid horseshoe prior: Degrees of freedom of ", 
           "the local priors must be a single positive number.")
   }
+  if (!isTRUE(df_global > 0)) {
+    stop2("Invalid horseshoe prior: Degrees of freedom of ", 
+          "the global prior must be a single positive number.")
+  }
   if (!isTRUE(scale_global > 0)) {
     stop2("Invalid horseshoe prior: Scale of the global ", 
           "prior must be a single positive number.")
   }
-  prior <- "normal(0, hs_local * hs_global)"
-  nlist(prior, df, scale_global)
+  att <- nlist(df, df_global, scale_global)
+  attributes(out)[names(att)] <- att
+  out
 }
 
+#' Set up a lasso prior in \pkg{brms}
+#' 
+#' Function used to set up a lasso prior for population-level effects 
+#' in \pkg{brms}. The function does not evaluate its arguments --
+#' it exists purely to help set up the model.
+#' 
+#' @param df Degrees of freedom of the chi-sqaure prior of the inverse tuning
+#'   parameter. Defaults to \code{1}.
+#' @param scale Scale of the lasso prior. Defaults to \code{1}.
+#'   
+#' @return A character string obtained by \code{match.call()} with
+#'   additional arguments.
+#'   
+#' @details See \code{\link[brms:set_prior]{set_prior}} for more details
+#'   about the lasso prior.
+#'   
+#' @seealso \code{\link[brms:set_prior]{set_prior}}
+#'   
+#' @examples 
+#' set_prior(lasso(df = 1, scale = 10))
+#' 
+#' @export
 lasso <- function(df = 1, scale = 1) {
-  # validate input for lasso prior
-  # Args:
-  #   df: degrees of freedom of the chi-square distribution 
-  #       of inv_lambda
+  out <- deparse(match.call())
   df <- round(as.numeric(df)[1], 5)
   scale <- round(as.numeric(scale)[1], 5)
   if (!isTRUE(df > 0)) {
@@ -1191,6 +1281,7 @@ lasso <- function(df = 1, scale = 1) {
     stop2("Invalid lasso prior: Scale of the Laplace ", 
           "priors must be a single positive number.")
   }
-  prior <- paste0("double_exponential(0, ", scale, " * lasso_inv_lambda)")
-  nlist(prior, df, scale)
+  att <- nlist(df, scale)
+  attributes(out)[names(att)] <- att
+  out
 }
