@@ -6,8 +6,6 @@ test_that("brm produces expected errors", {
                "missing in formula: 'b'")
   expect_error(brm(~ x + (1|g), dat), 
                "response variable is missing")
-  expect_error(brm(bf(y ~ exp(-x/a) + (1|g), a ~ 1, nl = TRUE), dat),
-               "Group-level terms cannot be specified")
   expect_error(brm(bf(y ~ a, nl = TRUE)),
                "No non-linear parameters specified")
   expect_error(brm(bf(y ~ a, a ~ 1, nl = TRUE), family = acat()),
@@ -46,10 +44,10 @@ test_that("brm produces expected errors", {
   # ordinal models
   expect_error(brm(rating ~ treat + period + carry + cse(treat) + (1|subject), 
                    data = inhaler, family = cratio("logit")), 
-               paste("Error occured for variables: treat"))
+               paste("Error occured for variables: 'treat'"))
   expect_error(brm(rating ~ treat + period + carry + monotonic(carry),
                    data = inhaler, family = cratio("logit")), 
-               paste("Error occured for variables: carry"))
+               paste("Error occured for variables: 'carry'"))
   
   # families and links
   expect_error(brm(y ~ x, dat, family = gaussian("logit")), 
