@@ -6,16 +6,16 @@ test_that("fitted helper functions run without errors", {
   fit <- add_samples(fit, "hu", dist = "beta", shape1 = 1, shape2 = 1)
   fit <- add_samples(fit, "zi", dist = "beta", shape1 = 1, shape2 = 1)
   fit <- add_samples(fit, "quantile", dist = "beta", shape1 = 2, shape2 = 1)
-  fit <- add_samples(fit, "nu", dist = "exp")
+  fit <- add_samples(fit, "xi", dist = "unif", min = -1, max = 0.5)
   draws <- brms:::extract_draws(fit)
-  eta <- brms:::linear_predictor(draws)
+  draws$mu <- brms:::linear_predictor(draws)
   nsamples <- nsamples(fit)
   nobs <- nobs(fit)
   
   # test preparation of truncated models
   draws$data$lb <- 0
   draws$data$ub <- 200
-  mu <- brms:::fitted_trunc(eta, draws)
+  mu <- brms:::fitted_trunc(draws)
   expect_equal(dim(mu), c(nsamples, nobs))
   
   # pseudo log-normal model
@@ -23,11 +23,16 @@ test_that("fitted helper functions run without errors", {
   expect_equal(dim(fitted(fit, summary = FALSE)), 
                c(nsamples, nobs))
   
-  # pseudo aysm_laplace model
+  # pseudo asym_laplace model
   fit$family <- asym_laplace()
   expect_equal(dim(fitted(fit, summary = FALSE)), 
                c(nsamples, nobs))
   
+  # pseudo gen_extreme_value model
+  fit$family <- gen_extreme_value()
+  expect_equal(dim(fitted(fit, summary = FALSE)), 
+               c(nsamples, nobs))
+  
   # pseudo weibull model
   fit$formula$pforms <- NULL
   fit$family <- weibull()
@@ -51,43 +56,44 @@ test_that("fitted helper functions run without errors", {
   expect_equal(dim(fitted(fit, summary = FALSE)), 
                c(nsamples, nobs))
   
-  # truncated continous models
+  # truncated continuous models
   draws$nu <- c(posterior_samples(fit, pars = "^nu$", as.matrix = TRUE))
   draws$shape <- c(posterior_samples(fit, pars = "^shape$", as.matrix = TRUE))
-  mu <- fitted_trunc_gaussian(eta, lb = 0, ub = 10, draws = draws)
+  mu <- fitted_trunc_gaussian(draws, lb = 0, ub = 10)
   expect_equal(dim(mu), c(nsamples, nobs))
   
-  mu <- fitted_trunc_student(eta, lb = -Inf, ub = 15, draws = draws)
+  mu <- fitted_trunc_student(draws, lb = -Inf, ub = 15)
   expect_equal(dim(mu), c(nsamples, nobs))
   
-  mu <- fitted_trunc_lognormal(eta, lb = 2, ub = 15, draws = draws)
-  expect_equal(dim(mu), c(nsamples, nobs))
-  exp_eta <- exp(eta)
-  mu <- fitted_trunc_gamma(exp_eta, lb = 1, ub = 7, draws = draws)
+  mu <- fitted_trunc_lognormal(draws, lb = 2, ub = 15)
   expect_equal(dim(mu), c(nsamples, nobs))
   
-  mu <- fitted_trunc_exponential(exp_eta, lb = 0, ub = Inf, draws = draws)
+  draws$mu <- exp(draws$mu)
+  mu <- fitted_trunc_gamma(draws, lb = 1, ub = 7)
   expect_equal(dim(mu), c(nsamples, nobs))
   
-  mu <- SW(fitted_trunc_weibull(exp_eta, lb = -Inf, ub = Inf, draws = draws))
+  mu <- fitted_trunc_exponential(draws, lb = 0, ub = Inf)
+  expect_equal(dim(mu), c(nsamples, nobs))
+  
+  mu <- SW(fitted_trunc_weibull(draws, lb = -Inf, ub = Inf))
   expect_equal(dim(mu), c(nsamples, nobs))
   
   # truncated discrete models
   data <- list(Y = sample(100, 10), trials = 1:10, N = 10)
   lb <- matrix(0, nrow = nsamples, ncol = nobs)
   ub <- matrix(100, nrow = nsamples, ncol = nobs)
-  mu <- fitted_trunc_poisson(exp_eta, lb = lb, ub = ub, draws = draws)
+  mu <- fitted_trunc_poisson(draws, lb = lb, ub = ub)
   expect_equal(dim(mu), c(nsamples, nobs))
   
-  mu <- fitted_trunc_negbinomial(exp_eta, lb = lb, ub = ub, draws = draws)
+  mu <- fitted_trunc_negbinomial(draws, lb = lb, ub = ub)
   expect_equal(dim(mu), c(nsamples, nobs))
   
-  mu <- fitted_trunc_geometric(exp_eta, lb = lb, ub = ub, draws = draws)
+  mu <- fitted_trunc_geometric(draws, lb = lb, ub = ub)
   expect_equal(dim(mu), c(nsamples, nobs))
   
   draws$data$trials <- 120
   lb <- matrix(-Inf, nrow = nsamples, ncol = nobs)
-  expit_eta <- ilink(eta, "logit")
-  mu <- fitted_trunc_binomial(expit_eta, lb = lb, ub = ub, draws = draws)
+  draws$mu <- ilink(draws$mu, "logit")
+  mu <- fitted_trunc_binomial(draws, lb = lb, ub = ub)
   expect_equal(dim(mu), c(nsamples, nobs))
-})
\ No newline at end of file
+})
